@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2008 University of California
+// Copyright (C) 2017 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -23,9 +23,7 @@
 #include <Carbon/Carbon.h>
 #include "filesys.h"
 #include "util.h"
-#if (defined(SANDBOX) && defined(_DEBUG))
-#include "SetupSecurity.h"
-#endif
+#include "mac_util.h"
 #include "sandbox.h"
 #endif
 
@@ -55,50 +53,6 @@
 bool s_bSkipExitConfirmation = false;
 
 
-#ifdef __WXMAC__
-
-// Set s_bSkipExitConfirmation to true if cancelled because of logging out or shutting down
-OSErr QuitAppleEventHandler( const AppleEvent *appleEvt, AppleEvent* reply, UInt32 refcon ) {
-    DescType            senderType;
-    Size                actualSize;
-    ProcessSerialNumber SenderPSN;
-    ProcessInfoRec      pInfo;
-    FSSpec              fileSpec;
-    OSStatus            anErr;
-
-    // Refuse to quit if a modal dialog is open.  
-    // Unfortunately, I know of no way to disable the Quit item in our Dock menu
-    if (wxGetApp().IsModalDialogDisplayed()) {
-        SysBeep(4);
-        return userCanceledErr;
-    }
-    
-    anErr = AEGetAttributePtr(appleEvt, keyAddressAttr, typeProcessSerialNumber,
-                                &senderType, &SenderPSN, sizeof(SenderPSN), &actualSize);
-
-    if (anErr == noErr) {
-        pInfo.processInfoLength = sizeof( ProcessInfoRec );
-        pInfo.processName = NULL;
-        pInfo.processAppSpec = &fileSpec;
-
-        anErr = GetProcessInformation(&SenderPSN, &pInfo);
-
-        // Consider a Quit command from our Dock menu as coming from this application
-        if ( (pInfo.processSignature != 'dock') && (pInfo.processSignature != 'BNC!') ) {
-            s_bSkipExitConfirmation = true; // Not from our app, our dock icon or our taskbar icon
-            // The following may no longer be needed under wxCocoa-3.0.0
-            wxGetApp().ExitMainLoop();  // Prevents wxMac from issuing events to closed frames
-        }
-    }
-    
-    wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxID_EXIT);
-    wxGetApp().GetFrame()->GetEventHandler()->AddPendingEvent(evt);
-    return noErr;
-}
-
-#endif
-
-
 DEFINE_EVENT_TYPE(wxEVT_RPC_FINISHED)
 
 IMPLEMENT_APP(CBOINCGUIApp)
@@ -152,16 +106,20 @@ bool CBOINCGUIApp::OnInit() {
     m_iBOINCMGRDisableAutoStart = 0;
     m_iShutdownCoreClient = 0;
     m_iDisplayExitDialog = 1;
+    m_iDisplayShutdownConnectedClientDialog = 1;
     m_iGUISelected = BOINC_SIMPLEGUI;
     m_bSafeMessageBoxDisplayed = 0;
+    m_bRunDaemon = true;
+    m_bNeedRunDaemon = true;
 
     // Initialize local variables
     int      iErrorCode = 0;
     int      iDesiredLanguageCode = 0;
     bool     bOpenEventLog = false;
     wxString strDesiredSkinName = wxEmptyString;
+#ifdef SANDBOX
     wxString strDialogMessage = wxEmptyString;
-    wxString strOldLanguageCode = wxEmptyString;
+#endif
     bool     success = false;
 
 
@@ -170,8 +128,6 @@ bool CBOINCGUIApp::OnInit() {
     wxSystemOptions::SetOption(wxT("msw.staticbox.optimized-paint"), 0);
 #endif
 #ifdef __WXMAC__
-    bool launchedFromLogin = false;
-    
     // In wxMac-2.8.7, default wxListCtrl::RefreshItem() does not work
     // so use traditional generic implementation.
     // This has been fixed in wxMac-2.8.8, but the Mac native implementation:
@@ -181,9 +137,6 @@ bool CBOINCGUIApp::OnInit() {
     wxSystemOptions::SetOption(wxT("mac.listctrl.always_use_generic"), 1);
 
     AEInstallEventHandler( kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)QuitAppleEventHandler), 0, false );
-
-    // Cache the current process serial number
-    GetCurrentProcess(&m_psnCurrentProcess);
 #endif
 
 
@@ -212,12 +165,16 @@ bool CBOINCGUIApp::OnInit() {
     m_pConfig->SetPath(wxT("/"));
     m_pConfig->Read(wxT("AutomaticallyShutdownClient"), &m_iShutdownCoreClient, 0L);
     m_pConfig->Read(wxT("DisplayShutdownClientDialog"), &m_iDisplayExitDialog, 1L);
+    m_pConfig->Read(wxT("DisplayShutdownConnectedClientDialog"), &m_iDisplayShutdownConnectedClientDialog, 1L);
     m_pConfig->Read(wxT("DisableAutoStart"), &m_iBOINCMGRDisableAutoStart, 0L);
     m_pConfig->Read(wxT("LanguageISO"), &m_strISOLanguageCode, wxT(""));
     m_pConfig->Read(wxT("GUISelection"), &m_iGUISelected, BOINC_SIMPLEGUI);
     m_pConfig->Read(wxT("EventLogOpen"), &bOpenEventLog);
+    m_pConfig->Read(wxT("RunDaemon"), &m_bRunDaemon, 1L);
 
-
+    // Detect if the daemon should be launched
+    m_bNeedRunDaemon = m_bNeedRunDaemon && m_bRunDaemon;
+    
     // Should we abort the BOINC Manager startup process?
     if (m_bBOINCMGRAutoStarted && m_iBOINCMGRDisableAutoStart) {
         return false;
@@ -319,19 +276,21 @@ bool CBOINCGUIApp::OnInit() {
 
 #ifdef SANDBOX
     // Make sure owners, groups and permissions are correct for the current setting of g_use_sandbox
+    //
+    // NOTE: GDB and LLDB can't attach to applications which are running as
+    // a different user or group.
+    // Normally, the Mac Development (Debug) builds do not define SANDBOX, so
+    // check_security() is never called. However, it is possible to use GDB
+    // or LLDB on sandbox-specific code, as long as the code is run as the
+    // current user (i.e., not as boinc_master or boinc_project), and the
+    // current user is a member of both groups boinc_master and boinc_project.
+    // However, this has not been thoroughly tested. Please see the comments
+    // in SetupSecurity.cpp and check_security.cpp for more details.
+    //
     char path_to_error[MAXPATHLEN];
     path_to_error[0] = '\0';
     
     if (!iErrorCode) {
-#if (defined(__WXMAC__) && defined(_DEBUG))     // TODO: implement this for other platforms
-        // GDB can't attach to applications which are running as a different user   
-        //  or group, so fix up data with current user and group during debugging
-        if (check_security(g_use_sandbox, true, NULL, 0)) {
-            CreateBOINCUsersAndGroups();
-            SetBOINCDataOwnersGroupsAndPermissions();
-            SetBOINCAppOwnersGroupsAndPermissions(NULL);
-        }
-#endif
         iErrorCode = check_security(
             g_use_sandbox, true, path_to_error, sizeof(path_to_error)
         );
@@ -342,11 +301,19 @@ bool CBOINCGUIApp::OnInit() {
         ShowApplication(true);
 
         if (iErrorCode == -1099) {
+#if (defined(__WXMAC__) && defined (_DEBUG))
             strDialogMessage.Printf(
-                _("You currently are not authorized to manage the client.\n\nTo run %s as this user, please:\n  - reinstall %s answering \"Yes\" to the question about\n     non-administrative users\n or\n  - contact your administrator to add you to the 'boinc_master'\n     user group."),
+                "To debug with sandbox security enabled, the current user\n"
+                "must be a member of both groups boinc_master and boinc_project."
+            );
+#else   // ! (defined(__WXMAC__) && defined (_DEBUG))
+            strDialogMessage.Printf(
+                _("You currently are not authorized to manage %s.\n\nTo run %s as this user, please:\n- reinstall %s answering \"Yes\" to the question about non-administrative users\n or\n- contact your administrator to add you to the 'boinc_master' user group."),
+                m_pSkinManager->GetAdvanced()->GetApplicationShortName().c_str(),
                 m_pSkinManager->GetAdvanced()->GetApplicationShortName().c_str(),
                 m_pSkinManager->GetAdvanced()->GetApplicationShortName().c_str()
             );
+#endif  // ! (defined(__WXMAC__) && defined (_DEBUG))
         } else {
             strDialogMessage.Printf(
                 _("%s ownership or permissions are not set properly; please reinstall %s.\n(Error code %d"),
@@ -399,31 +366,15 @@ bool CBOINCGUIApp::OnInit() {
 #endif
 
 #ifdef __WXMAC__
-    ProcessSerialNumber psn;
-    ProcessInfoRec pInfo;
-    OSStatus err;
-    
-    memset(&pInfo, 0, sizeof(pInfo));
-    pInfo.processInfoLength = sizeof( ProcessInfoRec );
-    err = GetProcessInformation(&m_psnCurrentProcess, &pInfo);
-    if (!err) {
-        psn = pInfo.processLauncher;
-        memset(&pInfo, 0, sizeof(pInfo));
-        pInfo.processInfoLength = sizeof( ProcessInfoRec );
-        err = GetProcessInformation(&psn, &pInfo);
-    }
-    // Don't open main window if we were started automatically at login
-    if (pInfo.processSignature == 'lgnw') {  // Login Window app
-        launchedFromLogin = true;
-        
-        // Prevent a situation where wxSingleInstanceChecker lock file
-        // from last login auto start (with same pid) was not deleted.
-        // This path must match that in DetectDuplicateInstance()
-        wxString lockFilePath = wxString(wxFileName::GetHomeDir() +
-                                            "/Library/Application Support/BOINC/" +
-                                            wxTheApp->GetAppName() +
-                                            '-' + wxGetUserId()
+    // Prevent a situation where wxSingleInstanceChecker lock file
+    // from last login auto start (with same pid) was not deleted.
+    // This path must match that in DetectDuplicateInstance()
+    wxString lockFilePath = wxString(wxFileName::GetHomeDir() +
+                                        "/Library/Application Support/BOINC/" +
+                                        wxTheApp->GetAppName() +
+                                        '-' + wxGetUserId()
                                         );
+    if (WasFileModifiedBeforeSystemBoot((char *)(const char*)lockFilePath.utf8_str())) {
         boinc_delete_file(lockFilePath.utf8_str());
     }
 #endif
@@ -476,9 +427,12 @@ bool CBOINCGUIApp::OnInit() {
     IdleTrackerAttach();
     
 #ifdef __WXMAC__
-    if (launchedFromLogin) {
-        m_bGUIVisible = false;
+    // Don't open main window if we were started automatically at login
+    // We are launched hidden if started from our login item (except if
+    // we had windows open at logout, the system "restores" them.)
+    m_bGUIVisible = IsApplicationVisible();
 
+    if (getTimeSinceBoot() < 30.) {
         // If the system was just started, we usually get a "Connection
         // failed" error if we try to connect too soon, so delay a bit.
         sleep(10);
@@ -617,7 +571,9 @@ void CBOINCGUIApp::SaveState() {
     m_pConfig->Write(wxT("LanguageISO"), m_strISOLanguageCode);
     m_pConfig->Write(wxT("AutomaticallyShutdownClient"), m_iShutdownCoreClient);
     m_pConfig->Write(wxT("DisplayShutdownClientDialog"), m_iDisplayExitDialog);
+    m_pConfig->Write(wxT("DisplayShutdownConnectedClientDialog"), m_iDisplayShutdownConnectedClientDialog);
     m_pConfig->Write(wxT("DisableAutoStart"), m_iBOINCMGRDisableAutoStart);
+    m_pConfig->Write(wxT("RunDaemon"), m_bRunDaemon);
 }
 
 
@@ -643,6 +599,7 @@ void CBOINCGUIApp::OnInitCmdLine(wxCmdLineParser &parser) {
 #if (defined(__WXMAC__) && defined(_DEBUG))
     parser.AddLongOption("NSDocumentRevisionsDebugMode", _("Not used: workaround for bug in XCode 4.2"));
 #endif
+    parser.AddSwitch("nd", "no-daemon", _("Not run the daemon"));
 }
 
 
@@ -717,6 +674,10 @@ bool CBOINCGUIApp::OnCmdLineParsed(wxCmdLineParser &parser) {
     if (hostNameSpecified && passwordSpecified) {
         m_bMultipleInstancesOK = true;
     }
+
+    if (parser.Found(wxT("no-daemon"))) {
+        m_bNeedRunDaemon = false;
+    }
     return true;
 }
 
@@ -765,6 +726,16 @@ void CBOINCGUIApp::DetectExecutableName() {
 
     // Store the root directory for later use.
     m_strBOINCMGRExecutableName = pszProg;
+#elif defined(__WXGTK__)
+    char path[PATH_MAX];
+    if (!get_real_executable_path(path, PATH_MAX)) {
+        // find filename component
+        char* name = strrchr(path, '/');
+        if (name) {
+            name++;
+            m_strBOINCMGRExecutableName = name;
+        }
+    }
 #endif
 }
 
@@ -786,6 +757,17 @@ void CBOINCGUIApp::DetectRootDirectory() {
 
     // Store the root directory for later use.
     m_strBOINCMGRRootDirectory = szPath;
+#elif defined(__WXGTK__)
+    char path[PATH_MAX];
+    if (!get_real_executable_path(path, PATH_MAX)) {
+        // find path component
+        char* name = strrchr(path, '/');
+        if (name) {
+            name++;
+            *name = '\0';
+            m_strBOINCMGRRootDirectory = path;
+        }
+    }
 #endif
 }
 
@@ -1057,6 +1039,7 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
     wxInt32          iLeft = 0;
     wxInt32          iHeight = 0;
     wxInt32          iWidth = 0;
+    bool             bWindowMaximized = false;
 
 
     // Create the new window
@@ -1070,6 +1053,7 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
             m_pConfig->Read(wxT("XPos"), &iLeft, 30);
             m_pConfig->Read(wxT("Width"), &iWidth, 800);
             m_pConfig->Read(wxT("Height"), &iHeight, 600);
+            m_pConfig->Read(wxT("WindowMaximized"), &bWindowMaximized, false);
             // Guard against a rare situation where registry values are zero
             if (iWidth < 50) iWidth = 800;
             if (iHeight < 50) iHeight = 600;
@@ -1177,6 +1161,9 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
             if (m_pFrame->IsIconized()) {
                 m_pFrame->Maximize(false);
             }
+            else if (BOINC_ADVANCEDGUI == iGUISelection && bWindowMaximized) {
+                m_pFrame->Maximize();
+            }
             m_pFrame->Raise();
 
 #ifdef __WXMSW__
